import { Action } from '@ngrx/store';
import { PlannedTerm } from '@app/core/models/planned-term';
import { DegreePlannerState } from '@app/degree-planner/state';

export enum PlanActionTypes {
	InitialPlanLoadResponse = '[Plan] Initial Load Response',
	ChangeVisiblePlanRequest = '[Plan] Change Visible Request',
	ChangeVisiblePlanResponse = '[Plan] Change Visible Response',
	ChangeCourseTermRequest = '[Plan] Change Course Term Request',
	ChangeCourseTermResponse = '[Plan] Change Course Term Response'
}

export class InitialPlanLoadResponse implements Action {
	public readonly type = PlanActionTypes.InitialPlanLoadResponse;
	constructor(public payload: DegreePlannerState) {}
}

export class ChangeVisiblePlanRequest implements Action {
	public readonly type = PlanActionTypes.ChangeVisiblePlanRequest;
	constructor(public payload: { newVisibleRoadmapId: number }) {}
}

export class ChangeVisiblePlanResponse implements Action {
	public readonly type = PlanActionTypes.ChangeVisiblePlanResponse;
	constructor(
		public payload: { visibleRoadmapId: number; visibleTerms: PlannedTerm[] }
	) {}
}

export class ChangeCourseTermRequest implements Action {
	public readonly type = PlanActionTypes.ChangeCourseTermRequest;
	constructor(
		public payload: {to: string, from: string, id: number}
	) {}
}

export class ChangeCourseTermResponse implements Action {
	public readonly type = PlanActionTypes.ChangeCourseTermResponse;
	constructor(
		public payload: {to: string, from: string, id: number}
	) {}
}
